var utils = {};
/**
 * Message Digest algorithm 5
 *
 * @param String string
 */
utils.MD5 = function (string) {
	function RotateLeft(lValue, iShiftBits) {
		return (lValue << iShiftBits) | (lValue >>> (32 - iShiftBits));
	}

	function AddUnsigned(lX, lY) {
		var lX4,
		lY4,
		lX8,
		lY8,
		lResult;
		lX8 = (lX & 0x80000000);
		lY8 = (lY & 0x80000000);
		lX4 = (lX & 0x40000000);
		lY4 = (lY & 0x40000000);
		lResult = (lX & 0x3FFFFFFF) + (lY & 0x3FFFFFFF);
		if (lX4 & lY4) {
			return (lResult^0x80000000^lX8^lY8);
		}
		if (lX4 | lY4) {
			if (lResult & 0x40000000) {
				return (lResult^0xC0000000^lX8^lY8);
			} else {
				return (lResult^0x40000000^lX8^lY8);
			}
		} else {
			return (lResult^lX8^lY8);
		}
	}

	function F(x, y, z) {
		return (x & y) | ((~x) & z);
	}
	function G(x, y, z) {
		return (x & z) | (y & (~z));
	}
	function H(x, y, z) {
		return (x^y^z);
	}
	function I(x, y, z) {
		return (y^(x | (~z)));
	}

	function FF(a, b, c, d, x, s, ac) {
		a = AddUnsigned(a, AddUnsigned(AddUnsigned(F(b, c, d), x), ac));
		return AddUnsigned(RotateLeft(a, s), b);
	};

	function GG(a, b, c, d, x, s, ac) {
		a = AddUnsigned(a, AddUnsigned(AddUnsigned(G(b, c, d), x), ac));
		return AddUnsigned(RotateLeft(a, s), b);
	};

	function HH(a, b, c, d, x, s, ac) {
		a = AddUnsigned(a, AddUnsigned(AddUnsigned(H(b, c, d), x), ac));
		return AddUnsigned(RotateLeft(a, s), b);
	};

	function II(a, b, c, d, x, s, ac) {
		a = AddUnsigned(a, AddUnsigned(AddUnsigned(I(b, c, d), x), ac));
		return AddUnsigned(RotateLeft(a, s), b);
	};

	function ConvertToWordArray(string) {
		var lWordCount;
		var lMessageLength = string.length;
		var lNumberOfWords_temp1 = lMessageLength + 8;
		var lNumberOfWords_temp2 = (lNumberOfWords_temp1 - (lNumberOfWords_temp1 % 64)) / 64;
		var lNumberOfWords = (lNumberOfWords_temp2 + 1) * 16;
		var lWordArray = Array(lNumberOfWords - 1);
		var lBytePosition = 0;
		var lByteCount = 0;
		while (lByteCount < lMessageLength) {
			lWordCount = (lByteCount - (lByteCount % 4)) / 4;
			lBytePosition = (lByteCount % 4) * 8;
			lWordArray[lWordCount] = (lWordArray[lWordCount] | (string.charCodeAt(lByteCount) << lBytePosition));
			lByteCount++;
		}
		lWordCount = (lByteCount - (lByteCount % 4)) / 4;
		lBytePosition = (lByteCount % 4) * 8;
		lWordArray[lWordCount] = lWordArray[lWordCount] | (0x80 << lBytePosition);
		lWordArray[lNumberOfWords - 2] = lMessageLength << 3;
		lWordArray[lNumberOfWords - 1] = lMessageLength >>> 29;
		return lWordArray;
	};

	function WordToHex(lValue) {
		var WordToHexValue = "",
		WordToHexValue_temp = "",
		lByte,
		lCount;
		for (lCount = 0; lCount <= 3; lCount++) {
			lByte = (lValue >>> (lCount * 8)) & 255;
			WordToHexValue_temp = "0" + lByte.toString(16);
			WordToHexValue = WordToHexValue + WordToHexValue_temp.substr(WordToHexValue_temp.length - 2, 2);
		}
		return WordToHexValue;
	};

	function Utf8Encode(string) {
		string = string.replace(/\r\n/g, "\n");
		var utftext = "";

		for (var n = 0; n < string.length; n++) {

			var c = string.charCodeAt(n);

			if (c < 128) {
				utftext += String.fromCharCode(c);
			} else if ((c > 127) && (c < 2048)) {
				utftext += String.fromCharCode((c >> 6) | 192);
				utftext += String.fromCharCode((c & 63) | 128);
			} else {
				utftext += String.fromCharCode((c >> 12) | 224);
				utftext += String.fromCharCode(((c >> 6) & 63) | 128);
				utftext += String.fromCharCode((c & 63) | 128);
			}

		}

		return utftext;
	};

	var x = Array();
	var k,
	AA,
	BB,
	CC,
	DD,
	a,
	b,
	c,
	d;
	var S11 = 7,
	S12 = 12,
	S13 = 17,
	S14 = 22;
	var S21 = 5,
	S22 = 9,
	S23 = 14,
	S24 = 20;
	var S31 = 4,
	S32 = 11,
	S33 = 16,
	S34 = 23;
	var S41 = 6,
	S42 = 10,
	S43 = 15,
	S44 = 21;

	string = Utf8Encode(string);

	x = ConvertToWordArray(string);

	a = 0x67452301;
	b = 0xEFCDAB89;
	c = 0x98BADCFE;
	d = 0x10325476;

	for (k = 0; k < x.length; k += 16) {
		AA = a;
		BB = b;
		CC = c;
		DD = d;
		a = FF(a, b, c, d, x[k + 0], S11, 0xD76AA478);
		d = FF(d, a, b, c, x[k + 1], S12, 0xE8C7B756);
		c = FF(c, d, a, b, x[k + 2], S13, 0x242070DB);
		b = FF(b, c, d, a, x[k + 3], S14, 0xC1BDCEEE);
		a = FF(a, b, c, d, x[k + 4], S11, 0xF57C0FAF);
		d = FF(d, a, b, c, x[k + 5], S12, 0x4787C62A);
		c = FF(c, d, a, b, x[k + 6], S13, 0xA8304613);
		b = FF(b, c, d, a, x[k + 7], S14, 0xFD469501);
		a = FF(a, b, c, d, x[k + 8], S11, 0x698098D8);
		d = FF(d, a, b, c, x[k + 9], S12, 0x8B44F7AF);
		c = FF(c, d, a, b, x[k + 10], S13, 0xFFFF5BB1);
		b = FF(b, c, d, a, x[k + 11], S14, 0x895CD7BE);
		a = FF(a, b, c, d, x[k + 12], S11, 0x6B901122);
		d = FF(d, a, b, c, x[k + 13], S12, 0xFD987193);
		c = FF(c, d, a, b, x[k + 14], S13, 0xA679438E);
		b = FF(b, c, d, a, x[k + 15], S14, 0x49B40821);
		a = GG(a, b, c, d, x[k + 1], S21, 0xF61E2562);
		d = GG(d, a, b, c, x[k + 6], S22, 0xC040B340);
		c = GG(c, d, a, b, x[k + 11], S23, 0x265E5A51);
		b = GG(b, c, d, a, x[k + 0], S24, 0xE9B6C7AA);
		a = GG(a, b, c, d, x[k + 5], S21, 0xD62F105D);
		d = GG(d, a, b, c, x[k + 10], S22, 0x2441453);
		c = GG(c, d, a, b, x[k + 15], S23, 0xD8A1E681);
		b = GG(b, c, d, a, x[k + 4], S24, 0xE7D3FBC8);
		a = GG(a, b, c, d, x[k + 9], S21, 0x21E1CDE6);
		d = GG(d, a, b, c, x[k + 14], S22, 0xC33707D6);
		c = GG(c, d, a, b, x[k + 3], S23, 0xF4D50D87);
		b = GG(b, c, d, a, x[k + 8], S24, 0x455A14ED);
		a = GG(a, b, c, d, x[k + 13], S21, 0xA9E3E905);
		d = GG(d, a, b, c, x[k + 2], S22, 0xFCEFA3F8);
		c = GG(c, d, a, b, x[k + 7], S23, 0x676F02D9);
		b = GG(b, c, d, a, x[k + 12], S24, 0x8D2A4C8A);
		a = HH(a, b, c, d, x[k + 5], S31, 0xFFFA3942);
		d = HH(d, a, b, c, x[k + 8], S32, 0x8771F681);
		c = HH(c, d, a, b, x[k + 11], S33, 0x6D9D6122);
		b = HH(b, c, d, a, x[k + 14], S34, 0xFDE5380C);
		a = HH(a, b, c, d, x[k + 1], S31, 0xA4BEEA44);
		d = HH(d, a, b, c, x[k + 4], S32, 0x4BDECFA9);
		c = HH(c, d, a, b, x[k + 7], S33, 0xF6BB4B60);
		b = HH(b, c, d, a, x[k + 10], S34, 0xBEBFBC70);
		a = HH(a, b, c, d, x[k + 13], S31, 0x289B7EC6);
		d = HH(d, a, b, c, x[k + 0], S32, 0xEAA127FA);
		c = HH(c, d, a, b, x[k + 3], S33, 0xD4EF3085);
		b = HH(b, c, d, a, x[k + 6], S34, 0x4881D05);
		a = HH(a, b, c, d, x[k + 9], S31, 0xD9D4D039);
		d = HH(d, a, b, c, x[k + 12], S32, 0xE6DB99E5);
		c = HH(c, d, a, b, x[k + 15], S33, 0x1FA27CF8);
		b = HH(b, c, d, a, x[k + 2], S34, 0xC4AC5665);
		a = II(a, b, c, d, x[k + 0], S41, 0xF4292244);
		d = II(d, a, b, c, x[k + 7], S42, 0x432AFF97);
		c = II(c, d, a, b, x[k + 14], S43, 0xAB9423A7);
		b = II(b, c, d, a, x[k + 5], S44, 0xFC93A039);
		a = II(a, b, c, d, x[k + 12], S41, 0x655B59C3);
		d = II(d, a, b, c, x[k + 3], S42, 0x8F0CCC92);
		c = II(c, d, a, b, x[k + 10], S43, 0xFFEFF47D);
		b = II(b, c, d, a, x[k + 1], S44, 0x85845DD1);
		a = II(a, b, c, d, x[k + 8], S41, 0x6FA87E4F);
		d = II(d, a, b, c, x[k + 15], S42, 0xFE2CE6E0);
		c = II(c, d, a, b, x[k + 6], S43, 0xA3014314);
		b = II(b, c, d, a, x[k + 13], S44, 0x4E0811A1);
		a = II(a, b, c, d, x[k + 4], S41, 0xF7537E82);
		d = II(d, a, b, c, x[k + 11], S42, 0xBD3AF235);
		c = II(c, d, a, b, x[k + 2], S43, 0x2AD7D2BB);
		b = II(b, c, d, a, x[k + 9], S44, 0xEB86D391);
		a = AddUnsigned(a, AA);
		b = AddUnsigned(b, BB);
		c = AddUnsigned(c, CC);
		d = AddUnsigned(d, DD);
	}

	var temp = WordToHex(a) + WordToHex(b) + WordToHex(c) + WordToHex(d);

	return temp.toLowerCase();
}

if (typeof atob == "undefined") {

	/**
	 * decode text string
	 *
	 * @param String encStr
	 *
	 * @return String
	 */
	var atob = function (encStr) {
		var base64s = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
		var bits;
		var decOut = "";
		var i = 0;
		for (; i < encStr.length; i += 4) {
			bits = (base64s.indexOf(encStr.charAt(i)) & 0xff) << 18 | (base64s.indexOf(encStr.charAt(i + 1)) & 0xff) << 12 | (base64s.indexOf(encStr.charAt(i + 2)) & 0xff) << 6 | base64s.indexOf(encStr.charAt(i + 3)) & 0xff;
			decOut += String.fromCharCode((bits & 0xff0000) >> 16, (bits & 0xff00) >> 8, bits & 0xff);
		}
		if (encStr.charCodeAt(i - 2) == 61) {
			return decOut.substring(0, decOut.length - 2);
		} else if (encStr.charCodeAt(i - 1) == 61) {
			return decOut.substring(0, decOut.length - 1);
		} else {
			return decOut;
		}
	};
}

if (typeof btoa == "undefined") {

	/**
	 * decode text string
	 *
	 * @param String encStr
	 *
	 * @return String
	 */
	var btoa = function (encStr) {

		var keyString = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";
		var output = "";
		var chr1,
		chr2,
		chr3,
		enc1,
		enc2,
		enc3,
		enc4;
		var i = 0;
		while (i < encStr.length) {
			chr1 = encStr.charCodeAt(i++);
			chr2 = encStr.charCodeAt(i++);
			chr3 = encStr.charCodeAt(i++);
			enc1 = chr1 >> 2;
			enc2 = ((chr1 & 3) << 4) | (chr2 >> 4);
			enc3 = ((chr2 & 15) << 2) | (chr3 >> 6);
			enc4 = chr3 & 63;
			if (isNaN(chr2)) {
				enc3 = enc4 = 64;
			} else if (isNaN(chr3)) {
				enc4 = 64;
			}
			output = output + keyString.charAt(enc1) + keyString.charAt(enc2) + keyString.charAt(enc3) + keyString.charAt(enc4);
		}
		return output;
	};
}

//Encodes a unicode string to UTF8 format.
utils.utf8Encode = function (str) {
	if (str != null) {
		code2utf = function (code) {
			chr = function (code) {
				return String.fromCharCode(code);
			};
			if (code < 128)
				return chr(code);
			if (code < 2048)
				return chr(192 + (code >> 6)) + chr(128 + (code & 63));
			if (code < 65536)
				return chr(224 + (code >> 12)) + chr(128 + ((code >> 6) & 63)) + chr(128 + (code & 63));
			if (code < 2097152)
				return chr(240 + (code >> 18)) + chr(128 + ((code >> 12) & 63)) + chr(128 + ((code >> 6) & 63)) + chr(128 + (code & 63));
		};

		_utf8Encode = function (str) {
			var utf8str = new Array();
			for (var i = 0; i < str.length; i++) {
				utf8str[i] = code2utf(str.charCodeAt(i));
			}
			return utf8str.join('');
		};

		var utf8str = new Array();
		var pos,
		j = 0;
		var tmpStr = '';
		while ((pos = str.search(/[^\x00-\x7F]/)) != -1) {
			tmpStr = str.match(/([^\x00-\x7F]+[\x00-\x7F]{0,10})+/)[0];
			utf8str[j++] = str.substr(0, pos);
			utf8str[j++] = _utf8Encode(tmpStr);
			str = str.substr(pos + tmpStr.length);
		}

		utf8str[j++] = str;
		return utf8str.join('');
	} else {
		return str;
	}

}

//Decodes a unicode string from UTF8 format.
utils.utf8Decode = function (str) {
	alert("utf8decode: " + str);
	var string = "";
	var i = 0;
	var c = c1 = c2 = 0;
	while (i < str.length) {
		c = str.charCodeAt(i);
		if (c < 128) {
			string += String.fromCharCode(c);
			i++;
		} else if ((c > 191) && (c < 224)) {
			c2 = str.charCodeAt(i + 1);
			string += String.fromCharCode(((c & 31) << 6) | (c2 & 63));
			i += 2;
		} else {
			c2 = str.charCodeAt(i + 1);
			c3 = str.charCodeAt(i + 2);
			string += String.fromCharCode(((c & 15) << 12) | ((c2 & 63) << 6) | (c3 & 63));
			i += 3;
		}
	}
	return string;
}

/**
 * Get params from locationSearch
 * @param String locationSearch
 *
 * @return Object
 */
utils.getParamFromLocationSearch = function (locationSearch) {
	if (!locationSearch || locationSearch == "")
		return null;

	var q = locationSearch.slice(1).split("&");

	var params = {};
	for (var i = 0, len = q.length; i < len; i++) {
		var kv = q[i].split("=");
		params[kv[0]] = unescape(kv[1]);
	}

	return params || null;
};

/**
 *
 * Parse the hash into an object
 * @param String hash
 *   pattern: [tab{files|webshares|favorites|account}:path]
 *            [tab{files|webshares|favorites|account}]
 *            [path]
 *
 * Returns a js object with tab and path attributes.
 */
utils.getParsedHashObject = function (hash) {
	var tokens = hash.replace(/^#/, "").split(":");
	var o = {
		tab : "",
		path : ""
	};
	if (tokens.length > 0) {
		if (tokens[0].search(/^files|^webshares|^favorites|^account/) > -1) {
			o.tab = tokens[0];
		}
		if (tokens.length == 1) {
			if (o.tab == "") {
				o.path = tokens[0];
			}
		} else {
			o.path = tokens[1];
		}
	}
	return o;
};

utils.setHash = function (hash) {
	document.locationHash = hash;
	document.location.hash = document.locationHash;
}

utils.sortXML = function (xmlDoc, query, sortByAttr, order) {

	var aryFind = new Array();
	sortByAttr = sortByAttr;
	xmlDoc = xmlDoc;
	query = query;
	order = order;
	alert("order " + order);
	alert("query " + query);
	alert("sortByAttr " + sortByAttr);

	if (order == "asc") {

		function compareAttr(a, b) {
			var sortA = a.sortByAttr.toLowerCase();
			var sortB = b.sortByAttr.toLowerCase();

			if (sortA < sortB)
				return -1;
			if (sortA > sortB)
				return 1;
			return 0;
		}

		$(xmlDoc.documentElement).find(query).each(function (value, fsFolder) {
			aryFind.push({
				xmlNode : fsFolder.cloneNode(true),
				sortByAttr : $(fsFolder).attr('name')
			});
		});

		if (aryFind && aryFind.length > 0) {
			aryFind.sort(compareAttr);

			var childItems = $(xmlDoc.documentElement).find(query);
			for (var i = 0, len = childItems.length; i < len; i++) {
				var oldNode = childItems[i];
				var newNode = aryFind[i].xmlNode;
				oldNode.parentNode.replaceChild(newNode, oldNode);
			}
		}
	} else if (order == "desc") {

		function compareAttribute(a, b) {
			var sortAA = a.sortByAttr.toLowerCase();
			var sortBB = b.sortByAttr.toLowerCase();

			if (sortAA < sortBB)
				return 1;
			if (sortAA > sortBB)
				return -1;
			return 0;
		}

		$(xmlDoc.documentElement).find(query).each(function (value, fsFolder) {
			aryFind.push({
				xmlNode : fsFolder.cloneNode(true),
				sortByAttr : $(fsFolder).attr('name')
			});
		});

		if (aryFind && aryFind.length > 0) {
			aryFind.sort(compareAttribute);

			var childItems = $(xmlDoc.documentElement).find(query);
			for (var i = 0, len = childItems.length; i < len; i++) {
				var oldNode = childItems[i];
				var newNode = aryFind[i].xmlNode;
				oldNode.parentNode.replaceChild(newNode, oldNode);
			}
		}
	}

	return xmlDoc
}

/**
 * remove files
 *
 * @param IXMLDOMDocument xmlDoc
 *
 * @return IXMLDOMDocument
 */
utils.removeFiles = function (xmlDoc) {
	if (!xmlDoc)
		return xmlDoc;

	var removeFile = {
		"settings.xml" : true
	};

	$(xmlDoc.documentElement).find(($.browser.webkit || $.browser.opera ? 'entry' : 'atom\\:entry')).each(function (value, entry) {
		if (removeFile[$(this).find(($.browser.webkit || $.browser.opera ? 'title' : 'atom\\:title')).text()]) {
			$(entry).remove();
		}
	});

	return xmlDoc;
}

/**
 * Get Storage Date and time
 *
 * @param String storageTime
 *
 * @return String
 */
utils.getStorageDatetime = function (storageTime) {
	//  alert("time: "+storageTime);
	var tokens = storageTime.split("T");
	var dateParts = tokens[0].split("-");
	//  alert("token 1: "+tokens[1]);
	var aux = tokens[1].slice(0, 8);
	var timeParts = aux.split(":");

	var utc = new Date();
	utc.setDate(1);
	utc.setFullYear(dateParts[0]);
	utc.setMonth(dateParts[1] - 1);
	utc.setDate(dateParts[2]);
	utc.setHours(timeParts[0]);
	utc.setMinutes(timeParts[1]);
	utc.setSeconds(timeParts[2]);

	var local = new Date();
	var utcDate = new Date();
	utcDate.setDate(1);
	utcDate.setFullYear(local.getUTCFullYear());
	utcDate.setMonth(local.getUTCMonth());
	utcDate.setDate(local.getUTCDate());
	utcDate.setHours(local.getUTCHours());
	utcDate.setMinutes(local.getUTCMinutes());
	utcDate.setSeconds(local.getUTCSeconds());
	var hourDiff = Math.round((local - utcDate) / 3600000);
	utc.setHours(utc.getHours() + hourDiff);

	var date = utc;
	var year = date.getFullYear();
	var month = date.getMonth() + 1;
	var day = date.getDate();
	var hour = date.getHours();
	var minutes = date.getMinutes();
	var seconds = date.getSeconds();
	return year + "-" + (month < 10 ? ("0" + month) : month) + "-" + (day < 10 ? ("0" + day) : day) + " " +
	(hour < 10 ? ("0" + hour) : hour) + ":" + (minutes < 10 ? ("0" + minutes) : minutes) + ":" + (seconds < 10 ? ("0" + seconds) : seconds);
}

/**
 * Format bytes to B ,KB, MB,GB
 *
 * @param Integer bytes
 *
 * @return String
 */
utils.formatBytes = function (bytes) {
	if (bytes < 1024) {
		return bytes + ' b';
	} else if (bytes < 1048576) {
		return Math.round(bytes / 1024) + ' KB';
	} else if (bytes < 1073741824) {
		return Math.round(bytes / 1048576) + ' MB';
	} else {
		return (Math.round((bytes / 1073741824) * 10) / 10) + ' GB';
	}
}

utils.getTotalResults = function (xmlDoc) {
	return parseInt($(xmlDoc.nodeType == 9 ? xmlDoc.documentElement : xmlDoc).children("[nodeName='os:totalResults']").text(), 10);
};

/**
 * Gets IXMLDOMNodes from IXMLDOMNode
 *
 * @param Object obj
 * @param IXMLDOMNode xmlNode
 *
 * @return Object
 */
utils.getChildNodesByObj = function (obj, xmlNode) {

	var childNodes = {},
	counter = 0;
	var i = 0,
	len = xmlNode.childNodes.length,
	childNode = null,
	childName = "";
	do {
		childNode = xmlNode.childNodes[i];
		childName = childNode.localName || childNode.baseName;
		if (childNode.nodeType == 1 && childName in obj) {
			if (childNodes[childName]) { // childNode already added, add a counter
				childNodes[childName + "_" + counter] = childNode;
			} else {
				childNodes[childName] = childNode;
			}
		}
	} while (++i < len);

	return childNodes;
}

utils.capitaliseFirstLetter = function (string) {
	return string.charAt(0).toUpperCase() + string.slice(1);
}

utils.timeToS = function (hour, min, sec) {
	var s;
	alert("hour: " + hour);
	alert("min: " + min);
	alert("sec: " + sec);
	s = ((parseInt(hour) * 60 * 60) + (parseInt(min) * 60) + parseInt(sec));
	alert("ms " + s);
	return s;
}
